
-- SurgeonGeneral by Brimmstone
-- Version 1.0
--	Initial Version

SurgeonGeneral = LibStub("AceAddon-3.0"):NewAddon("SurgeonGeneral", "AceConsole-3.0", "AceEvent-3.0")

local options = {
	name = "SurgeonGeneral",
	handler = SurgeonGeneral,
    type = "group",
    args = {
		Reset = {
			type = "execute",
			name = "Reset",
			desc = "Reset Window Position",
			func = "ResetWindowPosition",
			order = 1,
		},
		Verbose = {
			type = "toggle",
			name = "Verbose",
			desc = "Toggles Verbose Mode",
			set = "ToggleVerbose",
			get = "GetVerbose",
			order = 3,
		},
		Condense = {
			type = "toggle",
			name = "Condense Assignments",
			desc = "Multiple healers with the same assignment are grouped",
			set = "ToggleCondense",
			get = "GetCondense",
			order = 2,
		},
	},
}

local defaults = {
	profile = {
		Verbose = false,
		Condense = true,
	},
}

local VariablesLoaded

local AutoFillInProgress
local AutoFillCurrentUnit
local AutoFillCurrentUnitClassE
local AutoFillCurrentUnitClassL
local AutoFillCurrentID
local AutoFillTaint
local AutoFillCurrentSlot

local RaidNameList = {"Karazhan","Gruul's Lair","Magtheridon's Lair","Serpentshrine Cavern","Tempest Keep","Mount Hyjal","Black Temple","Sunwell Plateau"};
local BossNameList = {};
BossNameList.KZ = {"Attumen","Moroes","Maiden of Virtue","Opera Event","Nightbane","The Curator","Terestian Illhoof","Shade of Aran","Netherspite","Prince Malchezaar"}
BossNameList.GL = {"High King Maulgar","Gruul"}
BossNameList.ML = {"Magtheridon"}
BossNameList.SSC = {"Hydross the Unstable","The Lurker Below","Leotheras the Blind","Fathom Lord Karathress","Morogrim Tidewalker","Lasy Vashj"}
BossNameList.TK = {"Al'ar","Void Reaver","High Astromancer Solarian","Kael'thas Sunstrider"}
BossNameList.MH = {"Rage Winterchill","Anetheron","Kaz'rogal","Azgalor","Archimonde"}
BossNameList.BT = {"High Warlord Naj'entus","Supremus","Shade of Akama","Teron Gorefiend","Gurtogg Bloodboil","Reliquary of Souls","Mother Sharhaz","Illidari Council","Illidan Stormrage"};
BossNameList.SWP = {"Kalecgos","Brutallus","Felmyst","Eredar Twins","M'uru","Kil'jaeden"};

function SurgeonGeneral:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SurgeonGeneralDB", defaults, "Default")
		
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SurgeonGeneral", options);
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SurgeonGeneral", "SurgeonGeneral");
    self:RegisterChatCommand("sg", "ChatCommand");
	
	hooksecurefunc("NotifyInspect", SurgeonGeneral_NotifyInspect);
end

function SurgeonGeneral:ChatCommand(input)
	local x
	if not input or input:trim() == "" then
		SurgeonGeneralConfigFrame:Show();
		UIDropDownMenu_SetWidth(RaidNameDropdown,175);
		UIDropDownMenu_SetWidth(BossNameDropdown,175);
		for x = 1,10 do
			if getglobal("HealerClass"..x.."Texture"):GetTexture() == nil then
				UpdateHealerClass(x);
			end
		end
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(SurgeonGeneral, "sg", "SurgeonGeneral", input)
	end
end

function SurgeonGeneral:ResetWindowPosition()
	SurgeonGeneral:ClearAllPoints();
	SurgeonGeneral:SetPoint("CENTER","UIParent");
end

function SurgeonGeneral:ToggleVerbose()
	self.db.profile.Verbose = not self.db.profile.Verbose
end

function SurgeonGeneral:GetVerbose()
	return self.db.profile.Verbose;
end

function SurgeonGeneral:ToggleCondense()
	self.db.profile.Condense = not self.db.profile.Condense
end

function SurgeonGeneral:GetCondense()
	return self.db.profile.Condense;
end

function SurgeonGeneral:CopyBossFromTarget()
	local name;
	if (UnitExists("target")) then
		if (UnitIsPlayer("target")) then
			return;
		else
			name,_ = UnitName("target");
			getglobal("BossName"):SetText(name);
		end
	end
end

function SurgeonGeneral:OnLoad()
	AutoFillInProgress = 0;
	VariablesLoaded = false;
	local x;
	getglobal("Phase1Name"):SetText("Phase1");
	getglobal("Phase2Name"):SetText("Phase2");
	getglobal("Phase3Name"):SetText("Phase3");
	getglobal("Phase4Name"):SetText("Phase4");
	getglobal("Phase5Name"):SetText("Phase5");
	SurgeonGeneral:RegisterEvent("INSPECT_TALENT_READY","AutoFillInspectReady");
	SurgeonGeneral:RegisterEvent("VARIABLES_LOADED","OnEvent");
	getglobal("AutoFillButton"):SetScript("OnClick",self.AutoFillStart);
	getglobal("ReportButton"):SetScript("OnClick",self.StartReport);
	--getglobal("AutoFillBossButton"):SetScript("OnClick",self.CopyBossFromTarget);
end

function SurgeonGeneral:OnEvent(event)
	if(event == "VARIABLES_LOADED") then
		VariablesLoaded = true;
	end
end

function SetVar()
	--SurgeonGeneral:Print("Set Var");
end

function CalcPhases()
	local NumberOfPhases = 1;
	local k;
	k = 2;
	while k < 6 do
		if getglobal("Phase"..k.."Enable"):GetChecked() then
			NumberOfPhases = k;
			k = k + 1;
		else
			k = 6;
		end
	end
	if NumberOfPhases < 4 then
		Phase5Enable:Disable();
	else
		Phase5Enable:Enable();
	end
	
	if NumberOfPhases < 3 then
		Phase4Enable:Disable();
	else
		Phase4Enable:Enable();
	end
	
	if NumberOfPhases < 2 then
		Phase3Enable:Disable();
	else
		Phase3Enable:Enable();
	end
	
	return NumberOfPhases;
end

function PhaseNameChanged()
	-- placeholder
end

function ValidatePhaseNames()
	if (getglobal("Phase1Name"):GetText() == "") then
		getglobal("Phase1Name"):SetText("Phase1");
	end
	if (getglobal("Phase2Name"):GetText() == "") then
		getglobal("Phase2Name"):SetText("Phase2");
	end
	if (getglobal("Phase3Name"):GetText() == "") then
		getglobal("Phase3Name"):SetText("Phase3");
	end
	if (getglobal("Phase4Name"):GetText() == "") then
		getglobal("Phase4Name"):SetText("Phase4");
	end
	if (getglobal("Phase5Name"):GetText() == "") then
		getglobal("Phase5Name"):SetText("Phase5");
	end
end

function PhaseEnable_OnClick()
	CalcPhases();
end

function SurgeonGeneral:ReportToChannel(ReportChannel)
	local HealerMessage;
	local HealerClass;
	local j,k;
	local ReportChannelNumber;
	local ReportChannelName;
	
	ValidatePhaseNames();
	
	if (ReportChannel == "RAID") then
		ReportChannelNumber = nil;
		ReportChannelName = "RAID"
	else	
		ReportChannelNumber,_ = GetChannelName(ReportChannel);
		ReportChannelName = "CHANNEL"
		if(ReportChannelNumber == 0) then
			return;
		end
	end
	
	--SendChatMessage(" ",ReportChannelName,nil,ReportChannelNumber);
	if (UIDropDownMenu_GetText(BossNameDropdown) ~= nil) then
		SendChatMessage("Healing Assignments: "..UIDropDownMenu_GetText(BossNameDropdown),ReportChannelName,nil,ReportChannelNumber);
	else
		SendChatMessage("Healing Assignments",ReportChannelName,nil,ReportChannelNumber);
	end
	
	local AssignmentTable = {};	
	
	if CalcPhases() > 1 then
		HealerMessage = "Healer:  ";
		HealerMessage = HealerMessage .. getglobal("Phase1Name"):GetText();
		for j = 2,CalcPhases() do
			HealerMessage = HealerMessage .. " ~ " .. getglobal("Phase"..j.."Name"):GetText();
		end
		SendChatMessage(HealerMessage,ReportChannelName,nil,ReportChannelNumber);
	end
	
	for k = 1,10 do
		if not (getglobal("HealerName"..k):GetText() == "") then
			HealerMessage = ""
			HealerMessage = HealerMessage .. getglobal("Healer"..k.."Phase1"):GetText();
			for j = 2,CalcPhases() do
				HealerMessage = HealerMessage .. " ~ " .. getglobal("Healer"..k.."Phase"..j):GetText();
			end
						
			if (not self.db.profile.Condense) then
				HealerMessage = getglobal("HealerName"..k):GetText() .. ":  "..HealerMessage;
				SendChatMessage(HealerMessage,ReportChannelName,nil,ReportChannelNumber);
			else
				if AssignmentTable[HealerMessage] == nil then
					AssignmentTable[HealerMessage] = getglobal("HealerName"..k):GetText();
				else
					AssignmentTable[HealerMessage] = AssignmentTable[HealerMessage]..", "..getglobal("HealerName"..k):GetText();
				end
			end
		end
	end
	
	if (not self.db.profile.Condense) then
		return;
	end
	
	for index,value in pairs(AssignmentTable) do
		if (type(index) == "number") then
			index = tostring(index)
		elseif (type(index) == "boolean") then
			if (index) then
				index = "true"
			else
				index = "false"
			end
		end
		SendChatMessage(value..": "..index,ReportChannelName,nil,ReportChannelNumber);
	end
end

function SurgeonGeneral:StartReport()
	if getglobal("RaidChannelEnable"):GetChecked() then
		SurgeonGeneral:ReportToChannel("RAID");
	end
	
	if getglobal("CustomChannelEnable"):GetChecked() then
		SurgeonGeneral:ReportToChannel(getglobal("CustomChannelName"):GetText());
	end
end

function SurgeonGeneral:VerbosePrint(msg,color)
	if (self.db.profile.Verbose) then
		if color ~= nil then
			DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Surgeon General: |cff"..color..msg.."|r");
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Surgeon General: |cffffffff"..msg.."|r");
		end
	end
end

function SurgeonGeneral:ForcePrint(msg,color)
	if color ~= nil then
		DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Surgeon General: |cff"..color..msg.."|r");
	else
		DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Surgeon General: |cffffffff"..msg.."|r");
	end
end

function SurgeonGeneral:AutoFillStart()
	if (not UnitInRaid("player")) then
		SurgeonGeneral:ForcePrint("You are not in a raid","ffff00");
		AutoFillInProgress = 0;
		return;
	end
	
	SurgeonGeneral:VerbosePrint("Auto Fill Starting...");
	
	AutoFillInProgress = 1;
	AutoFillCurrentID = 0;
	AutoFillCurrentSlot = 1;
	
	SurgeonGeneral:AutoFillNext();	
end

function SurgeonGeneral:AutoFillNext()
	if (AutoFillInProgress == 1) then
		while (AutoFillCurrentID < GetNumRaidMembers()) do
		
			-- break function if Autofill terminated while in this loop.
			if (AutoFillInProgress == 0) then
				return;
			end
			
			AutoFillCurrentID = AutoFillCurrentID + 1;
			AutoFillCurrentUnit = "raid"..AutoFillCurrentID;
			AutoFillCurrentUnitClassL,AutoFillCurrentUnitClassE = UnitClass(AutoFillCurrentUnit);			
			if ((AutoFillCurrentUnitClassE == "WARRIOR") or (AutoFillCurrentUnitClassE == "ROGUE") or (AutoFillCurrentUnitClassE == "HUNTER") or (AutoFillCurrentUnitClassE == "MAGE") or (AutoFillCurrentUnitClassE == "WARLOCK")) then
				-- If it's not a healing class, go to next unit
				SurgeonGeneral:VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." is not a healing class.");
			elseif UnitIsUnit("player",AutoFillCurrentUnit) then
				-- If it's me, just call the function to get my info (You can't inspect yourself).
				SurgeonGeneral:VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." is me.");
				SurgeonGeneral:AutoFillGetTalents();
			elseif (CheckInteractDistance(AutoFillCurrentUnit,1)) then
				-- If it's somebody else and they're within inspect range, get their info.
				SurgeonGeneral:VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." is a "..AutoFillCurrentUnitClassL..".  Inspecting...");
				NotifyInspect(AutoFillCurrentUnit);
				return;
			else
				-- If they're not within range, just skip them and move along
				SurgeonGeneral:ForcePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." was out of inspection range.");
			end
		end
		if (AutoFillInProgress == 1) then
			SurgeonGeneral:AutoFillStop();
		end
	end
end

function SurgeonGeneral:AutoFillStop()
	AutoFillInProgress = 0;
	SurgeonGeneral:ForcePrint("Auto Fill Complete");
end

function SurgeonGeneral_NotifyInspect(unit)
	if (AutoFillInProgress == 0) then
		return;
	elseif (unit == AutoFillCurrentUnit) then
		AutoFillTaint = 0;
	else
		AutoFillTaint = 1;
		SurgeonGeneral:VerbosePrint("Inspect Taint!","ff0000");
		NotifyInspect(AutoFillCurrentUnit);
	end
end

function SurgeonGeneral:AutoFillGetTalents()
	local HealingTalents;
	local HealingTree;
	local HealingTreeName;
		
	if (AutoFillInProgress == 0) then
		return;
	elseif (AutoFillTaint == 1) then
		return;
	end
	
	-- Set appropriate healing tree ID for class
	if (AutoFillCurrentUnitClassE == "PALADIN") then
		HealingTree = 1;
	elseif (AutoFillCurrentUnitClassE == "PRIEST") then
		HealingTree = 2;
	elseif (AutoFillCurrentUnitClassE == "SHAMAN") then
		HealingTree = 3;
	elseif (AutoFillCurrentUnitClassE == "DRUID") then
		HealingTree = 3;
	else
		--  Not a healing class... should not have been inspected, but trapping anyways.
		return;
	end
	
	if (UnitIsUnit("player",AutoFillCurrentUnit)) then
		HealingTreeName,_,HealingTalents,_ = GetTalentTabInfo(HealingTree,false);
	else
		HealingTreeName,_,HealingTalents,_ = GetTalentTabInfo(HealingTree,true);
	end
	
	-- Assuming if you call yourself a healer, you've put at least 36 talents in your healing tree.
	SurgeonGeneral:VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." has "..HealingTalents.." in the "..HealingTreeName.." tree.");
	if (HealingTalents > 35) then
		local HealerName,_ = UnitName(AutoFillCurrentUnit);
		getglobal("HealerName"..AutoFillCurrentSlot):SetText(HealerName);
		AutoFillCurrentSlot = AutoFillCurrentSlot + 1;
	end
	
	if ((AutoFillCurrentSlot < 11) and (AutoFillCurrentID < GetNumRaidMembers())) then
		SurgeonGeneral:AutoFillNext();
	else
		SurgeonGeneral:AutoFillStop();
	end
end

function SurgeonGeneral:AutoFillInspectReady()
	SurgeonGeneral:AutoFillGetTalents();
end

function HealerNameChange(healer)
	local id = tonumber(string.sub(healer:GetName(),11))
	UpdateHealerClass(id);
end

function UpdateHealerClass(icon)
	local _,englishClass = UnitClass(getglobal("HealerName"..icon):GetText());
	if englishClass ~= nil then
		getglobal("HealerClass"..icon.."Texture"):SetTexture("Interface\\Addons\\SurgeonGeneral\\Icons\\"..englishClass);
	else
		getglobal("HealerClass"..icon.."Texture"):SetTexture(nil);
	end
end

function EditBox_OnEscapePressed()
	SurgeonGeneralConfigFrame:Hide()
end

local info = {};

function RaidNameDropdown_OnLoad()
	if (VariablesLoaded == false) then
		return;
	end;
	
	local x;
	local List = RaidNameList;
	for x=1,getn(List) do
		info.text = List[x];
		info.value = x;
		info.owner = this:GetParent();
		info.func = function() RaidName_OnSelect() end;
		info.checked = nil;
		UIDropDownMenu_AddButton(info);
	end
end

function RaidName_OnSelect()
	if (UIDropDownMenu_GetSelectedValue(this.owner) ~= this.value) then
		UIDropDownMenu_SetSelectedValue(this.owner, this.value);
		UIDropDownMenu_ClearAll(BossNameDropdown);
		BossNameDropdown_OnLoad();
	end
	SurgeonGeneral:ForcePrint("OnSelect: " .. this.value .. " " .. UIDropDownMenu_GetText(RaidNameDropdown));
end

function BossNameDropdown_OnLoad()
	if (VariablesLoaded == false) then
		return;
	end;
	
	local x;
	local List = {};
	
	List = select(UIDropDownMenu_GetSelectedValue(RaidNameDropdown),BossNameList.KZ, BossNameList.GL, BossNameList.ML, BossNameList.SSC, BossNameList.TK, BossNameList.MH, BossNameList.BT, BossNameList.SWP);
		
	for x=1,getn(List) do
		info.text = List[x];
		info.value = x;
		info.owner = this:GetParent();
		info.hasarrow = true;
		info.func = function() BossName_OnSelect() end;
		info.checked = nil;
		UIDropDownMenu_AddButton(info);
	end
	
	if(UIDropDownMenu_GetSelectedValue(BossNameDropdown) == nil) then
		UIDropDownMenu_SetSelectedValue(BossNameDropdown,1)
		UIDropDownMenu_SetText(BossNameDropdown,List[1])
	end
end

function BossName_OnSelect()
	UIDropDownMenu_SetSelectedValue(this.owner, this.value);
	SurgeonGeneral:ForcePrint("OnSelect: " .. this.value .. " " .. UIDropDownMenu_GetText(BossNameDropdown));
end

--[[function SurgeonGeneral:BossNameDropdown_Initialize()
	UIDropDownMenu_Initialize(BossNameDropdown,BossNameDropdown_Setup);
end

function SurgeonGeneral:BossNameDropdown_OnLoad()
	SurgeonGeneral:BossNameDropdown_Initialize();
end
]]--